from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest
from asap3.md.velocitydistribution import *

#set_verbose(1)

atoms = FaceCenteredCubic(directions=((1,0,0), (0,1,0), (0,0,1)),
                          size=(15,15,15), symbol="Cu", pbc=True)
atoms.set_calculator(EMT())

atoms.get_forces()
atoms.get_forces()
MaxwellBoltzmannDistribution(atoms, 300 * units.kB)
atoms.get_forces()
atoms.get_forces()

atoms = FaceCenteredCubic(directions=((1,0,0), (0,1,0), (0,0,1)),
                          size=(15,15,15), symbol="Cu", pbc=True)
atoms.set_calculator(EMT())
s = atoms.get_stress()
print()
print("Stress:", s)
s = atoms.get_stress()
print()
print("Stress:", s)
MaxwellBoltzmannDistribution(atoms, 300 * units.kB)
s = atoms.get_stress()
print()
print("Stress:", s)
s = atoms.get_stress()
print()
print("Stress:", s)
MaxwellBoltzmannDistribution(atoms, 300 * units.kB)
s = atoms.get_stress()
print()
print("Stress:", s)
s = atoms.get_stress()
print()
print("Stress:", s)

atoms = FaceCenteredCubic(directions=((1,0,0), (0,1,0), (0,0,1)),
                          size=(15,15,15), symbol="Cu", pbc=True)
atoms.set_calculator(EMT())
s = atoms.get_stress()
atoms.get_forces()
atoms.get_forces()
MaxwellBoltzmannDistribution(atoms, 300 * units.kB)
atoms.get_forces()
atoms.get_forces()

atoms = FaceCenteredCubic(directions=((1,0,0), (0,1,0), (0,0,1)),
                          size=(15,15,15), symbol="Cu", pbc=True)
atoms.set_calculator(EMT())
atoms.get_stresses()
atoms.get_stresses()
MaxwellBoltzmannDistribution(atoms, 300 * units.kB)
atoms.get_stresses()
atoms.get_stresses()

print()
print()
print("No crash: Test passes succesfully!")



"""Usage: python SpecialEMT.py [init]

Used to test if the EMT potential still produces the same energies,
forces and stresses as it used to.  It tests all elements with the
standard EMT parameters, including alloys, and also the special
parameters for Ruthenium and for CuMg and CuZr metallic glasses.

Call it the first time with the "init" argument to generate a data file.

Subsequent calls calculate the same quantities, but compare them with the
database.
"""
from __future__ import print_function


from asap3 import *
import pickle
import os, sys, time
from asap3.testtools import ReportTest
from ase.lattice.cubic import *
from ase.lattice.hexagonal import *
from ase.lattice.compounds import *

print_version(1)
datafile = "SpecialEMT.pickle"
CuMgdat = "../doc/EMT-MgCu-glass.dat"
CuZrdat = "../doc/EMT-CuZr-glass.dat"

if sys.version_info[0] >= 3:
    picklehack = {'encoding': 'latin1'}
else:
    picklehack = {}
    
if len(sys.argv) > 1:
    if len(sys.argv) == 2 and sys.argv[1] == "init":
        init = True
    else:
        print(__doc__, file=sys.stderr)
        sys.exit(1)
else:
    init = False

if init:
    data = {}
else:
    try:
        data = pickle.load(open(datafile, "rb"), **picklehack)
    except IOError:
        print("\n\n", __doc__, "\n\n", file=sys.stderr)
        raise

def check(a, key):
    global data
    print("Testing", key)
    expected_n = data[key+"_n"]
    expected_e = data[key+"_e"]
    expected_f = data[key+"_f"]
    e = a.get_potential_energies()
    f = a.get_forces().ravel()
    ReportTest("Number of atoms (%s)" % (key,), len(a), expected_n, 0)
    ReportTest("Energies (%s)" % (key,), max(np.fabs(e-expected_e)), 0.0, 1e-8)
    ReportTest("Forces (%s)" % (key,), max(np.fabs(f-expected_f)), 0.0, 1e-8)

def generate(a, key):
    global data
    e = a.get_potential_energies()
    f = a.get_forces().ravel()
    n = len(a)
    data[key+"_n"] = n
    data[key+"_e"] = e
    data[key+"_f"] = f

elements = ["Al", "Ni", "Cu", "Pd", "Ag", "Pt", "Au"]
print("Checking single elements")
for e in elements:
    atoms = FaceCenteredCubic(size=(4,4,4), symbol=e, pbc=False)
    atoms.set_calculator(EMT())
    if init:
        generate(atoms, e)
    else:
        check(atoms, e)

print("Checking B2 alloys")
for i in range(1, len(elements)):
    e1 = elements[i]
    e2 = elements[i-1]
    atoms = B2(size=(4,4,4), symbol=(e1,e2), latticeconstant=3.5, pbc=False)
    atoms.set_calculator(EMT())
    if init:
        generate(atoms, e1+e2)
    else:
        check(atoms, e1+e2)

print("Checking Ruthenium")
atoms = HexagonalClosedPacked(directions=[[1,0,-1,0],[0,1,-1,0],[0,0,0,1]],
                              size=(4,4,4), symbol="Ru", pbc=False)
atoms.set_calculator(EMT(EMThcpParameters()))
if init:
    generate(atoms, "Ru")
else:
    check(atoms, "Ru")

print("Checking metallic glasses")
for e1, e2 in (("Cu", "Mg"), ("Cu", "Zr")):
    atoms = B2(size=(4,4,4), symbol=(e1,e2), latticeconstant=3.5, pbc=False)
    atoms.set_calculator(EMT(EMTMetalGlassParameters()))
    if init:
        generate(atoms, e1+e2)
    else:
        check(atoms, e1+e2)


if init:
    f = open(datafile, "wb")
    dump(data, f)
    f.close()
    print("\nData file initialized.")
    
    



"""The ASAP version number is specified in this file.

This file sets the __version__ variable to ASAP's version number.
The variable is imported by the main Asap module.  Furthermore, this module
prints the version number when executed, this is used by the makefile.
"""
from __future__ import print_function


__version__ = "3.11.9"


if __name__ == '__main__':
    print(__version__)
    



"Test the RawRadialDistributionFunction function."
from __future__ import print_function

from numpy import *
from asap3 import *
from asap3.analysis import RadialDistributionFunction
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import *

print_version(1)

# testtypes: latticeconst, maxRDF, bins, useEMT
testtypes = ((3.6, 6.001, 100, False),
             (3.7, 6.001, 100, True),
             (3.6, 15.001, 100, False),
             (3.65, 15.001, 100, True))

for latconst, maxrdf, nbins, withemt in testtypes:

    atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], symbol="Cu",
                              size=(10,10,10), latticeconstant=latconst)
    natoms = len(atoms)
    ReportTest("Number of atoms", natoms, 4000, 0)

    if withemt:
        atoms.set_calculator(EMT())
        print(atoms.get_potential_energy())

    rdf = RadialDistributionFunction(atoms, maxrdf, nbins)
    z = atoms.get_atomic_numbers()[0]

    globalrdf = rdf.get_rdf()
    localrdf = rdf.get_rdf(elements=(z,z))

    print(globalrdf)

    ReportTest("Local and global RDF are identical",
               max(abs(globalrdf - localrdf)), 0.0, 1e-6)
        
    shellpop = [12, 6, 24, 12, 24, -1]
    shell = [sqrt(i+1.0)/sqrt(2.0) for i in range(6)]
    print(shell)
    print(shellpop)
    n = 0

    dr = maxrdf/nbins

    for i in range(nbins):
        if (i+1)*dr >= shell[n] * latconst:
            if shellpop[n] == -1:
                print("Reached the end of the test data")
                break
            rho = len(atoms) / atoms.get_volume()
            expected = shellpop[n] / (4 * pi * ((i+0.5) * dr)**2 * dr * rho)
            maxerr = shellpop[n] / (8 * pi * ((i+0.5) * dr)**3 * rho) * 0.02
            print("Shell", n+1, globalrdf[i], expected, maxerr, (globalrdf[i] - expected)/maxerr)
            ReportTest(("Shell %d (%d)" % (n+1, i)), globalrdf[i],
                       expected, maxerr)
            n += 1
        else:
            ReportTest(("Between shells (%d)" % (i,)), globalrdf[i], 0, 0)


ReportTest.Summary()




"""Test OpenKIM Lennard Jones potential, including parameter modifications."""

from __future__ import print_function, division
from asap3 import *
from ase.build import bulk
import numpy as np
from asap3.testtools import ReportTest

model = 'LennardJones612_UniversalShifted__MO_959249795837_003'

if OpenKIMsupported:
    try:
        calc = OpenKIMcalculator(model)
    except AsapError as oops:
        if oops.args[0].startswith('Failed to initialize OpenKIM model'):
            print("OpenKIM model {} not installed - skipping test.".format(openkimmodel))
            calc = None
        else:
            raise

if OpenKIMsupported and calc is not None:

    p = calc.parameters
    # Get some parameters
    epsilon_He = p.get_parameter('epsilons', 'ut')[1,1]
    sigma_He = p.get_parameter('sigmas', 'ut')[1,1]
    # Set a parameter
    p['shift'] = 0

    helium = bulk("He", 'fcc', np.sqrt(2) * 2**(1/6) * sigma_He).repeat((5,5,5))
    helium.set_calculator(calc)

    e = helium.get_potential_energy() / len(helium)
    ReportTest("Helium energy", e, -8.24646294 * epsilon_He, 1e-6)

    calc2 = OpenKIMcalculator(model)
    eps = calc2.parameters.get_parameter('epsilons', 'ut')
    eps[1,1] = 2 * epsilon_He
    calc2.parameters.set_parameter('epsilons', eps, 'ut')
    calc2.parameters['shift'] = 0

    helium.set_calculator(calc2)
    e2 = helium.get_potential_energy() / len(helium)

    ReportTest("Modified Helium energy", e2, 2*e, 1e-6)


    ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from ase.visualize.fieldplotter import *

clean = True
plotlog = "fieldplotter.log"

atoms = FaceCenteredCubic(size=(50,50,5), symbol="Ag")

def xy(a=atoms):
    r = atoms.get_positions()
    return r[:,0] + r[:,1]

plotter = FieldPlotter(atoms, xy, verbose=1)
plotter.set_dimensions((110,100))
plotter.set_data_range('plot')
plotter.set_output(PostScriptFile("fieldplot"))
plotter.set_output(GifFile("fieldplot"))
plotter.set_log(plotlog)
plotter.plot()

if clean:
    for name in (plotlog, "fieldplot0000.ps", "fieldplot0000.gif"):
        print("deleting", name)
        os.unlink(name)





from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from numpy import *
from asap3.testtools import ReportTest

print_version(1)
#set_verbose(1)

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(6,6,6),
                          symbol="Cu", pbc=(1,1,0))
atoms.set_calculator(EMT())
ecorrect = atoms.get_potential_energy()

atoms2 = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(6,6,6),
                          symbol="Cu", pbc=(0,0,0))
atoms2.set_pbc((1,1,0))
atoms2.set_calculator(EMT())
e1= atoms2.get_potential_energy()
ReportTest("e1 correct", e1, ecorrect, 0.001)

atoms.set_pbc((0,1,0))
atoms.set_pbc((1,1,0))
e2 = atoms.get_potential_energy()
ReportTest("e2 correct", e2, ecorrect, 0.001)

print("Setting pbc")
atoms.set_pbc((0,1,0))
print("Calculating energy")
dummy =  atoms.get_potential_energy()
assert(fabs(dummy - ecorrect) > 1.0)
atoms.set_pbc((1,1,0))
e3 = atoms.get_potential_energy()
ReportTest("e3 correct", e3, ecorrect, 0.001)

atoms.set_pbc((1,1,0))
atoms.set_positions(atoms.get_positions())
e4 = atoms.get_potential_energy()
ReportTest("e4 correct", e4, ecorrect, 0.001)

print("Repeating tests with an atom outside the unit cell.")
for coordinate in (0,1,2):
    print("Using coordinate number", coordinate)

    atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(6,6,6),
                              symbol="Cu", pbc=(1,1,0))
    r = atoms.get_positions()
    uc = atoms.get_cell()
    r[-1,coordinate] = uc[coordinate,coordinate] * 1.51
    atoms.set_positions(r)
    atoms.set_calculator(EMT())
    ecorrect = atoms.get_potential_energy()

    atoms.set_pbc((0,1,0))
    atoms.set_pbc((1,1,0))
    e2 = atoms.get_potential_energy()
    ReportTest("e2 correct", e2, ecorrect, 0.001)

    atoms.set_pbc((0,1,0))
    dummy =  atoms.get_potential_energy()
    assert(fabs(dummy - ecorrect) > 1.0)
    atoms.set_pbc((1,1,0))
    e3 = atoms.get_potential_energy()
    ReportTest("e3 correct", e3, ecorrect, 0.001)

    atoms.set_pbc((1,1,0))
    atoms.set_positions(atoms.get_positions())
    e4 = atoms.get_potential_energy()
    ReportTest("e4 correct", e4, ecorrect, 0.001)


    
ReportTest.Summary()



### Effective medium theory calculator. ###
# encoding: utf-8

# Written by: Rasmus E. Christiansen, s072162
# Version 1.02

# Imported Packages
import sys
import numpy
from math import sqrt,exp
from ase.data import atomic_numbers, chemical_symbols
from ase.units import Bohr
from asap3 import FullNeighborList

# Gobal parameters 
parameters = {
#          E0     s0    V0     eta2    kappa   lambda  n0      Lattice type
#          eV     bohr  eV     bohr^-1 bohr^-1 bohr^-1 bohr^-3 
    'H':  (-2.21, 0.71, 2.132, 1.652,  2.790,  1.892,  0.00547, 'dimer'),
    'Al': (-3.28, 3.00, 1.493, 1.240,  2.000,  1.169,  0.00700, 'fcc'),
    'Cu': (-3.51, 2.67, 2.476, 1.652,  2.740,  1.906,  0.00910, 'fcc'),
    'Ag': (-2.96, 3.01, 2.132, 1.652,  2.790,  1.892,  0.00547, 'fcc'),
    'Au': (-3.80, 3.00, 2.321, 1.674,  2.873,  2.182,  0.00703, 'fcc'),
    'Ni': (-4.44, 2.60, 3.673, 1.669,  2.757,  1.948,  0.01030, 'fcc'),
    'Pd': (-3.90, 2.87, 2.773, 1.818,  3.107,  2.155,  0.00688, 'fcc'),
    'Pt': (-5.85, 2.90, 4.067, 1.812,  3.145,  2.192,  0.00802, 'fcc'),
    'C':  (-1.97, 1.18, 0.132, 3.652,  5.790,  2.892,  0.01322, 'dimer'),
    'N':  (-4.97, 1.18, 0.132, 2.652,  3.790,  2.892,  0.01222, 'dimer'),
    'O':  (-2.97, 1.25, 2.132, 3.652,  5.790,  4.892,  0.00850, 'dimer')}

beta = 1.809    # Calculated from the following equation
                # beta = ((16 * Pi) / 3)^(1/3) / 2^(1/2)

# The "largest" element possibly supported by the calculator (this is determined by the list chemical_symbols).
NumElle = len(chemical_symbols)


class EMT:
    """ This class is an implementation of the revised edition of the Effective Medium Theory approach of calculating the energy of a given FCC crystal system. The functional form of the equations used can be found in ******* """
    def __init__(self,Params=None,ZeroPoint=1):
        """ Initializes the EMT object. The input Params is used to specify userdefined parameters and the input 
            Zeropoint is used to specify whether the potential energy measured realtive to E0 = Ecoh (ZeroPoint = 1) 
            or E0 = 0 (ZeroPoint = 0). """
        # Secures that the calculator is initialized correctly the first time it is used.
        self.energy = None
        self.ZeroPoint = ZeroPoint
        # If no parameters have been specified when creating the EMT object the default parameters are used.
        if Params is None:
            self.parameters = parameters
        else:
            self.parameters = Params

    def initialize(self, atoms):
        """ Method which initializes the EMT calculator by defining all needed values used in the
            calculations. """
        # A list, Z, of the element type for each atom in the system is defined:
        self.Z = atoms.get_atomic_numbers()
        # The number of atoms are calculated: 
        self.N = len(self.Z)        
        # Lists of the values for eta2, kappa, Seq, E0, V0, n0 and L (lambda) for the element types Z are 
        # defined:
        # The "largest" element number is corrected with regards to the method of counting in python.
        self.eta2 = numpy.zeros(NumElle)
        self.kappa = numpy.zeros(NumElle)
        self.Seq = numpy.zeros(NumElle)
        self.E0 = numpy.zeros(NumElle)
        self.V0 = numpy.zeros(NumElle)
        self.L = numpy.zeros(NumElle)
        self.n0 = numpy.zeros(NumElle)
        for i in range(NumElle):
            if chemical_symbols[i] in self.parameters:
                self.eta2[i] = self.parameters[chemical_symbols[i]][3] / Bohr
                self.kappa[i] = self.parameters[chemical_symbols[i]][4] / Bohr
                self.Seq[i] = self.parameters[chemical_symbols[i]][1] * Bohr
                self.E0[i] = self.parameters[chemical_symbols[i]][0]
                self.V0[i] = self.parameters[chemical_symbols[i]][2]
                self.L[i] = self.parameters[chemical_symbols[i]][5] / Bohr
                self.n0[i] = self.parameters[chemical_symbols[i]][6] / (Bohr**3)
        
        # Calculation of the X*X-arrays:    
        # r_cut; X*X-array of vaules for the cutoff length for the atompair of type (Z,Z') 
        # sigmaaRCUT; X*X-array of values for sigmaa evaluated in r_cut[Z,Z']
        # sigmabRCUT; X*X-array of values for sigmab evaluated in r_cut[Z,Z']
        # dsigmaadrRCUT; X*X-array of values for the deriviative of sigmaa evaluated in r_cut[Z,Z']
        # dsigmabdrRCUT; X*X-array of values for the deriviative of sigmab evaluated in r_cut[Z,Z']
        # chi; X*X-array of values for chi for the atompair of type (Z,Z')

        # The cutoff distances are calculated using that the lattice constant, a0 = sqrt(2)*beta*s0:
        self.r_cut = numpy.zeros([NumElle,NumElle])

        for i in range(NumElle):
            for j in range(NumElle):
                # Check to see if the two elements considered are defined or not. Only calculating an 
                # r_cut if both are. r_cut[Z,Z'] is calculated as the cutoff distance for the the one of 
                # elements Z,Z' which has the largest a0.
                if self.Seq[i] and self.Seq[j] != 0:
                    self.r_cut[i,j] = (1./2. * (sqrt(3. / 2.) + sqrt(4. / 2.)) * (sqrt(2) * beta) * 
                                      max(self.Seq[i],self.Seq[j]))

        
        ### Calculations for sigmaaRCUT, sigmabRCUT, d_sigmaadrRCUT and d_sigmabdrRCUT ###

        self.dsigmaadrRCUT = numpy.zeros([NumElle,NumElle])
        self.dsigmabdrRCUT = numpy.zeros([NumElle,NumElle])
        self.sigmaaRCUT = numpy.zeros([NumElle,NumElle])
        self.sigmabRCUT = numpy.zeros([NumElle,NumElle])

        for i in range(NumElle):
            for j in range(NumElle):
                # Check to see if r_cut[i,j] is defined for this pair of elements. r_cut[i,j] == 0 means 
                # that it is not defined.
                if self.r_cut[i,j] != 0:
                    self.sigmaaRCUT[i,j] = (numpy.exp(self.eta2[j] * 
                                           (-self.r_cut[i,j] + self.Seq[j] * beta)))
                    self.sigmabRCUT[i,j] = (numpy.exp(self.kappa[j] * 
                                           (-self.r_cut[i,j] / beta + self.Seq[j])))
                    self.dsigmaadrRCUT[i,j] = -self.eta2[j] * self.sigmaaRCUT[i,j]
                    self.dsigmabdrRCUT[i,j] = -self.kappa[j] / beta * self.sigmabRCUT[i,j]
        
        
        ### Calculations for chi[Z,Z'] ###

        self.chi = numpy.zeros([NumElle,NumElle])

        for i in range(NumElle):
            for j in range(NumElle):
                # Check to see if the elements i,j are defined.
                if self.n0[i] and self.n0[j] != 0:
                    self.chi[i,j] = self.n0[i] / self.n0[j]


        ### Calculations of gamma1 and gamma2 ###

        # Four (3 x NumElle)-arrays for lambda_1,2 (named L_1,2) and sigmaa_1,2 are calculated with the distance, 
        # r_ij = (beta * Seq, sqrt(2) * beta * Seq, sqrt(3) * beta * Seq) for all supportet elements. 
        # where Z = Z'.
        
        # The NumberNearestNeighbours variable is set to the number of nearest neighbors included in the model.
        NumberNearestNeighbours = 3
        # arrays for lambda and sigmaa are initialized 
        L_1_Z = numpy.zeros([NumberNearestNeighbours,NumElle])
        L_2_Z = numpy.zeros([NumberNearestNeighbours,NumElle])
        sigmaa_Z = numpy.zeros([NumberNearestNeighbours,NumElle])
        sigmab_Z = numpy.zeros([NumberNearestNeighbours,NumElle])
        # The values for each are calculated for each neighbour distance
        for i in range(NumberNearestNeighbours):
            L_1_Z[i] = (self.dsigmaadrRCUT[range(NumElle),range(NumElle)] *
                       ((sqrt(1 + i) * beta * self.Seq) - self.r_cut[range(NumElle),range(NumElle)]) +
                        self.sigmaaRCUT[range(NumElle),range(NumElle)])

            L_2_Z[i] = (self.dsigmabdrRCUT[range(NumElle),range(NumElle)] *
                       ((sqrt(1 + i) * beta * self.Seq) - self.r_cut[range(NumElle),range(NumElle)]) +
                        self.sigmabRCUT[range(NumElle),range(NumElle)])
              
            sigmaa_Z[i] = numpy.exp(self.eta2 * (-(sqrt(1 + i) * beta * self.Seq) + self.Seq * beta))
            sigmab_Z[i] = numpy.exp(self.kappa * (-(sqrt(1 + i) * self.Seq) + self.Seq))

        
        # The factor (self.Seq/self.Seq) is an array of zeros and ones and is only used to secure that only 
        # the elements which are actually defined in "parameters" gives a gamma_1,2 different from zero.
        self.gamma1 = ((self.Seq/self.Seq) * 
                      (12 * (sigmaa_Z[0] - L_1_Z[0]) + 
                       6 * (sigmaa_Z[1] - L_1_Z[1]) + 
                       24 * (sigmaa_Z[2] - L_1_Z[2]) ))
        self.gamma2 = ((self.Seq/self.Seq) * 
                      (12 * (sigmab_Z[0] - L_2_Z[0]) + 
                       6 * (sigmab_Z[1] - L_2_Z[1]) + 
                       24 * (sigmab_Z[2] - L_2_Z[2]) ))

        ### Construction of a Full Neighborlist for the system of atoms, 
        self.nbList = FullNeighborList(self.r_cut.max(),atoms)

        ### Initialization of the variables holding the forces on and energy of the atoms ###
        self.forces = None
        self.energy = None
        

    def NeighborList_rcutReduced(self,i):
        """ Method which makes sure that only the neighboratoms within the correct cutoff for the involved 
        element types Z,Z' are included in the calculations by modifying the output of the FullNeighborList
        function. """

        # Relavant data about the neighbor atom, j, for atom i which can possible give a contribution are 
        # selected
        (other_j,r_ij,rsq) = self.nbList.get_neighbors(i)

        # The neighbor atoms which will actually give a contribution to the energy, based on the individual 
        # cutoff distances between atom i of type Z and atom j of type Z', are selected.

        # The neighbor atoms which fullfill the condition are chosen
        keep = numpy.sqrt(rsq) <= self.r_cut[self.Z[i],self.Z[other_j]]
        
        # The lists of data about the neighbor atoms are updated
        if len(keep) != 0:
            return (other_j[keep],r_ij[keep],rsq[keep])
        else:
            # nbList returned empty lists, but we cannot index a shape (0,3) array (r_ij)
            # with an empty list (bug in numpy?).
            return (other_j,r_ij,rsq)

    
    def update(self, atoms):
        """ This method is called by the atoms object to which the calculator is attached, it secures that the 
            energy (and/or force) of the system is recalculated if this is required. """
        need_calc = False
        if (self.energy is None or len(self.Z) != len(atoms) or (self.Z != atoms.get_atomic_numbers()).any()):
            # The calculator is initialized with regards to the atoms object.
            self.initialize(atoms)
            need_calc = True
        elif (self.positions != atoms.get_positions()).any():
            # The atoms object has not changed enough for the calculator to need a reinitialization but a 
            # new calculation of the value for the energies are still needed.
            need_calc = True
        if need_calc:
            self.positions = atoms.get_positions()
            self.nbList.check_and_update(atoms)
            self.energy = self.calculate_Energy()
            self.forces = self.calculate_Force()


    # Returns the energy of the atoms (the method calculates the energy first if needed be)
    def get_potential_energy(self, atoms):
         self.update(atoms)
         return self.energy

    # Returns the forces on the atoms (the method calculates the forces first if needed be)
    def get_forces(self, atoms):
        self.update(atoms)
        return self.forces.copy()

    def get_stress(self, atoms):
        raise NotImplementedError('No stresses implemented')


    ########## ENERGY Calculations ##########

    ### sigma_1,2 ###

    
    def calculate_sigma12(self):
        """ Calculates and returns sigma_1 and sigma_2. """
        # The N-arrays for sigma_1,2 are initialized
        sigma_1 = numpy.zeros(self.N)
        sigma_2 = numpy.zeros(self.N)

       
        for i in range(self.N):
            # The numbers of the neighbor atoms, the relative position vectors and length of the position 
            # vectors squared between atom i and the neighbor atoms j are defined in three arrays. 
            (other_j,r_ij,rsq) = self.NeighborList_rcutReduced(i)
            
            
            # The values for the linear subtracktion functions evaluated at norm(r_ij,2) for all the atom 
            # pairs, [i,other_j], are calculated.
            L_1_i = (self.dsigmaadrRCUT[self.Z[i],self.Z[other_j]] *
                    (numpy.sqrt(rsq) - self.r_cut[self.Z[i],self.Z[other_j]]) +
                     self.sigmaaRCUT[self.Z[i],self.Z[other_j]])
            L_2_i = (self.dsigmabdrRCUT[self.Z[i],self.Z[other_j]] *
                    (numpy.sqrt(rsq) - self.r_cut[self.Z[i],self.Z[other_j]]) +
                     self.sigmabRCUT[self.Z[i],self.Z[other_j]])
                       
            # sigmaa_i and sigmab_i are evaluated at norm(r_ij,2) for all the atom pairs, [i,other_j]. 
            sigmaa_i = (numpy.exp(self.eta2[self.Z[other_j]] * 
                       (-numpy.sqrt(rsq) + self.Seq[self.Z[other_j]] * beta)))
            sigmab_i = (numpy.exp(self.kappa[self.Z[other_j]] * 
                       (-numpy.sqrt(rsq) / beta + self.Seq[self.Z[other_j]])))

            #if (i == 1):
            #    print sigmaa_i

            # The values of sigma_1_i and sigma_2_i are calculated for
            # the atom i. Where max(a,b) is introduced in order to
            # secure a none zero minimumvalue for sigma_1 so the
            # following calculations wont result in an error.
            sigma_1[i] = max( pow(10,-9) , (self.chi[self.Z[i],self.Z[other_j]] * (sigmaa_i - L_1_i)).sum() )
            sigma_2[i] = (self.chi[self.Z[i],self.Z[other_j]] * (sigmab_i - L_2_i)).sum()

        #TESTER#
        #print sigma_1[:10]
        
        
        return (sigma_1,sigma_2)


    ### s_i ###

    def calculate_s(self,sigma_1):
        """ Calculates and returns an N-array containing the neutrality sphere radii, s, for the atoms of the 
        system is calculated."""    
        return self.Seq[self.Z] - numpy.log(sigma_1 / self.gamma1[self.Z]) / (beta * self.eta2[self.Z])
 

    ### E_tot ###

    
    def calculate_Energy_function(self,s,sigma_2):
        """ Calculates and returns the total energy of the system using s and sigma_2. """
        # Calculation of the N-array containing the cohesive energy for each of the N atoms.
        E_c = ( self.E0[self.Z] * (self.L[self.Z] * (s - self.Seq[self.Z]) + 1) * 
                numpy.exp(-self.L[self.Z] * (s - self.Seq[self.Z])) ) - self.E0[self.Z] * self.ZeroPoint

        # Calculation of the N-array containing the atomic sphere correction energy for each of the N atoms.
        E_as = (6 * ( self.V0[self.Z] * numpy.exp(-self.kappa[self.Z] * (s - self.Seq[self.Z])) 
                -self.V0[self.Z] * sigma_2 / self.gamma2[self.Z] ) )

        # Calculation of the total energy
        return (E_c + E_as).sum()


    ### Final Energy Calculator ###

    def calculate_Energy(self):
        """ Calculates and returnes the energy of the atoms in the atom object to which the 
            EMT calculator is attached. The calculations are done using the following methods, 
            also defined in EMT.py: calculate_sigma12(self), calculate_s(self,sigma_1), 
            calculate_Energy_function(self,s,sigma_2). """
        (sigma_1,sigma_2) = self.calculate_sigma12()
        s = self.calculate_s(sigma_1)
    
        # The total energy is calculated and returned
        return self.calculate_Energy_function(s,sigma_2)




    ########## FORCE Calculations ##########


    ### dsdsigma_1 ###

    
    def calculate_dsdsigma_1(self,sigma_1):
        """ Calculates and returns dsdsigma_1 using sigma_1. """
        # An N-array containing the the deriviative of neutrality sphere radii, s, with regards to sigma_1 for 
        # the atoms of the system is calculated.    
        dsdsigma_1 = -1 / (beta * self.eta2[self.Z] * sigma_1)

        return dsdsigma_1


    ### dE_cids, dE_asds, dE_asdsigma_2 ###

    def calculate_Deriviative_of_Energy(self,s):
        """ Calculates and returns the deriviatives of E_cs and E_as with regards to s and sigma_2. """
        # Calculation of the N-array containing the deriviative of the cohesive energy with regards to s for 
        # each of the N atoms.
        dE_cds = -( self.E0[self.Z] * self.L[self.Z] * self.L[self.Z] * 
                    numpy.exp(-self.L[self.Z] * (s - self.Seq[self.Z])) * (s - self.Seq[self.Z]) )

        # Calculation of the N-array containing the deriviative of the atomic sphere correction energy with 
        # regards to s for each of the N atoms.
        dE_asds = -6 * self.kappa[self.Z] * self.V0[self.Z] * numpy.exp(-self.kappa[self.Z] * (s - self.Seq[self.Z]))

        # Calculation of the N-array containing the deriviative of the atomic sphere correction energy with 
        # regards to sigma_2 for each of the N atoms.
        dE_asdsigma_2 = -6 * self.V0[self.Z] / (self.gamma2[self.Z])

        return (dE_cds,dE_asds,dE_asdsigma_2)


    ### F_kalpha ###
    def calculate_Force_function(self,dE_cds,dE_asds,dE_asdsigma_2,dsdsigma_1):
        """ Calculates the force on all k atoms in the three directions {x,y,z} representet by alpha. """ 
            
        # An array for the force is initialized
        F = numpy.zeros([self.N,3])

        for k in range(self.N):
            # The atoms interacting with atom k are selected.
            (other_i,r_ki,rsq) = self.NeighborList_rcutReduced(k)
            #print other_i
            #print r_ki
            #print numpy.sqrt(rsq)
            
            # The values for dr_ijdr_kalpha are calculated for the relevant atoms, k and other_i.
            dr_kidr_kalpha = r_ki / numpy.sqrt(rsq)[:,numpy.newaxis]
            
            ## The force on the k'th atom caused by the atoms, other_i's, interactions with k are calculated ##

            # The values for dsigmaa_idr_ij and dsigmab_idr_ij are calculated with regards to the k'th atom
            sigmaa_k = numpy.exp(self.eta2[self.Z[other_i]] * 
                       (-numpy.sqrt(rsq) + self.Seq[self.Z[other_i]] * beta))
            sigmab_k = numpy.exp(self.kappa[self.Z[other_i]] * 
                       (-numpy.sqrt(rsq) / beta + self.Seq[self.Z[other_i]]))
            
            dsigmaa_kdr_ki = (-self.eta2[self.Z[other_i]] * sigmaa_k)[:,numpy.newaxis]
            dsigmab_kdr_ki = (-self.kappa[self.Z[other_i]] / beta * sigmab_k)[:,numpy.newaxis]
            
            # Values for dL_1idr_ij and dL_2idr_ij are calculated with regards to the k'th atom
            dL_1kdr_ki = self.dsigmaadrRCUT[self.Z[k],self.Z[other_i]][:,numpy.newaxis]
            dL_2kdr_ki = self.dsigmabdrRCUT[self.Z[k],self.Z[other_i]][:,numpy.newaxis]
            
            # First the value of dsigma1_idr_kaplha and dsigma2_idr_kaplha are calculated for the k'th atom
            dsigma1_kdr_kalpha = ( self.chi[self.Z[k],self.Z[other_i]][:,numpy.newaxis] * 
                                   (dsigmaa_kdr_ki - dL_1kdr_ki) * dr_kidr_kalpha ).sum(axis=0)
            dsigma2_kdr_kalpha = ( self.chi[self.Z[k],self.Z[other_i]][:,numpy.newaxis] * 
                                   (dsigmab_kdr_ki - dL_2kdr_ki) * dr_kidr_kalpha ).sum(axis=0)
            
            """ TJEK DER SKAL FJERNES SENERE """
            assert len(dsigma1_kdr_kalpha) == 3
            """ TJEK DER SKAL FJERNES SENERE """

            # The contribution to the force on atom k from the k'th atoms interaction with the other_i atoms is 
            # calculated
            F[k] = (dE_cds[k] * dsdsigma_1[k] * dsigma1_kdr_kalpha +
                    dE_asds[k] * dsdsigma_1[k] * dsigma1_kdr_kalpha + 
                    dE_asdsigma_2[k] * dsigma2_kdr_kalpha)
            

            # The values for dsigmaa_idr_ij and dsigmab_idr_ij are calculated with regards to the atoms other_i 
            # where j = k for all other_i (thus we only need one value of dsigmaa_idr_ij).
            sigmaa_i = numpy.exp(self.eta2[self.Z[k]] * (-numpy.sqrt(rsq) + self.Seq[self.Z[k]] * beta))
            sigmab_i = numpy.exp(self.kappa[self.Z[k]] * (-numpy.sqrt(rsq) / beta + self.Seq[self.Z[k]]))
            
            dsigmaa_idr_ik = (-self.eta2[self.Z[k]] * sigmaa_i)[:,numpy.newaxis]
            dsigmab_idr_ik = (-self.kappa[self.Z[k]] / beta * sigmab_i)[:,numpy.newaxis]
            
            # Values for dL_1idr_ij and dL_2idr_ij are calculated with regards to the atoms other_i 
            # where j = k for all other_i.
            dL_1idr_ik = self.dsigmaadrRCUT[self.Z[other_i],self.Z[k]][:,numpy.newaxis]
            dL_2idr_ik = self.dsigmabdrRCUT[self.Z[other_i],self.Z[k]][:,numpy.newaxis]
            
            # First the value of dsigma1_idr_kaplha and dsigma2_idr_kaplha are calculated with regards to the atoms 
            # other_i where j are only the atom k for all other_i. (thus the sum only has one element for all other_i.
            # which results in the calculations leading to an [other_i,3]-array. 
            dsigma1_idr_kalpha = (self.chi[self.Z[other_i],self.Z[k]][:,numpy.newaxis] * 
                                  (dsigmaa_idr_ik - dL_1idr_ik) * (dr_kidr_kalpha) )
            dsigma2_idr_kalpha = (self.chi[self.Z[other_i],self.Z[k]][:,numpy.newaxis] * 
                                  (dsigmab_idr_ik - dL_2idr_ik) * (dr_kidr_kalpha) )
            
            # The contribution to the force on atom k from the other_i atoms interaction with the k'th atom is now 
            # calculated
            F[k] += (dE_cds[other_i][:,numpy.newaxis] * dsdsigma_1[other_i][:,numpy.newaxis] * dsigma1_idr_kalpha +
                     dE_asds[other_i][:,numpy.newaxis] * dsdsigma_1[other_i][:,numpy.newaxis] * dsigma1_idr_kalpha + 
                     dE_asdsigma_2[other_i][:,numpy.newaxis] * dsigma2_idr_kalpha).sum(axis=0)

            """ TJEK DER SKAL FJERNES SENERE """
            assert len(F[k]) == 3
            """ TJEK DER SKAL FJERNES SENERE """

        return F


    ### Final Force Calculator ###

    def calculate_Force(self):
        """ Calculates and returnes the force acting on each of the atoms in the atoms object to which the 
            EMT calculator is attached. These calculations are done using the following methods, 
            also defined in EMT.py: calculate_sigma12(self), calculate_s(self,sigma_1), calculate_dsdsigma_1
            (self,sigma_1), calculate_Deriviative_of_Energy(self,s) and calculate_Force_function
            (self,dE_cds,dE_asds,dE_asdsigma_2,dsdsigma_1) """
        (sigma_1,sigma_2) = self.calculate_sigma12()
        s = self.calculate_s(sigma_1)
        dsdsigma_1 = self.calculate_dsdsigma_1(sigma_1)
        (dE_cds,dE_asds,dE_asdsigma_2) = self.calculate_Deriviative_of_Energy(s)
    
        # The force is calculated and returned
        return self.calculate_Force_function(dE_cds,dE_asds,dE_asdsigma_2,dsdsigma_1)

    
 



"""Test the FixAtoms constraint and the Subset filter."""
from __future__ import print_function

from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.md.velocitydistribution import MaxwellBoltzmannDistribution
from asap3.md.verlet import VelocityVerlet
from asap3.constraints import Filter, FixAtoms
from ase.constraints import FixAtoms as ASE_FixAtoms
from asap3.testtools import ReportTest
from asap3.mpi import world
import numpy as np

debug = 0
if debug == 1:
    DebugOutput("parallelconstraints%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

print_version(1)
#set_verbose(1)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

if ismaster:
    init = FaceCenteredCubic(size=(10,10,10), symbol='Cu', pbc=False)
    z = init.get_positions()[:,2]
    fixedatoms = np.less(z, 0.501*z.max())
    print(len(init), sum(fixedatoms))
    MaxwellBoltzmannDistribution(init, 6000*units.kB)
    init.set_tags(fixedatoms)
else:
    init = None

print()
print("Running simulation with Filter")
atoms1 = MakeParallelAtoms(init, cpulayout)
atoms1.arrays['r_init'] = atoms1.get_positions()
atoms1.set_calculator(EMT())
atoms1a = Filter(atoms1, mask=np.logical_not(atoms1.get_tags()))

dyn = VelocityVerlet(atoms1a, 3*units.fs)
dyn.run(100)

print()
print("Running simulation with Asap's FixAtoms")
atoms2 = MakeParallelAtoms(init, cpulayout)
atoms2.arrays['r_init'] = atoms2.get_positions()
atoms2.set_calculator(EMT())
atoms2.set_constraint(FixAtoms(mask=atoms2.get_tags()))

dyn = VelocityVerlet(atoms2, 3*units.fs)
dyn.run(100)

print()
print("Running Langevin simulation with Asap's FixAtoms")
atoms4 = MakeParallelAtoms(init, cpulayout)
atoms4.arrays['r_init'] = atoms4.get_positions()
atoms4.set_calculator(EMT())
atoms4.set_constraint(FixAtoms(mask=atoms4.get_tags()))

dyn = Langevin(atoms4, 3*units.fs, 3000*units.kB, 0.01)
dyn.run(100)


print()
sanity = [[atoms1, "Verlet + Filter"],
          [atoms2, "Verlet + Asap's FixAtoms"],
          [atoms4, "Langevin + Asap's FixAtoms"],
          ]
for a, label in sanity:
    print(world.rank, "Sanity check, %s:", (label,))
    r_init = a.arrays['r_init']
    ok = (a.get_positions() == r_init) + np.logical_not(a.get_tags())[:,np.newaxis]
    if ok.all():
        print(world.rank, "  Stationary atoms have not moved: OK")
    else:
        raise RuntimeError("Stationary atoms have moved (%s)" % (label,))
    ok = (a.get_positions() != r_init) + a.get_tags()[:,np.newaxis]
    if ok.all():
        print(world.rank, "  Mobile atoms have moved: OK")
    else:
        raise RuntimeError("Mobile atoms have not moved (%s)" % (label,))
    

print("ALL TESTS SUCCEEDED!")






from __future__ import print_function
import numpy as np

from ase import Atoms, units
from ase.data import atomic_numbers
from asap3 import Morse, PropertyNotImplementedError
from ase.test import must_raise

#from calculators.morse import MorsePotential as Morse

# Define constants and calculator
elements = np.array([atomic_numbers['Ru'], atomic_numbers['Ar']])
epsilon = np.array([[5.720, 0.092], [0.092, 0.008]])
alpha = np.array([[1.475, 2.719], [2.719, 1.472]])
rmin = np.array([[2.110, 2.563], [2.563, 4.185]])
rcut = rmin.max() + 6.0 / alpha.min()

calc = Morse(elements, epsilon, alpha, rmin)
atoms = Atoms(['Ar', 'Ar'], [[0.0, 0.0, 0.0], [rcut + 1.0, 0.0, 0.0]])
atoms.center(vacuum=10.0)
atoms.set_pbc(True)
atoms.set_calculator(calc)

# This should not fail
energy = atoms.get_potential_energy()

#This should fail
with must_raise(PropertyNotImplementedError):
    stress = atoms.get_stress()




"""Test local crystal structure analysis (CNA and Coordination number)."""
from __future__ import print_function

from asap3 import *
from asap3.analysis import FullCNA, RestrictedCNA, CoordinationNumbers
from asap3.testtools import ReportTest
from asap3.mpi import world
from ase.lattice import bulk
import numpy as np

debug = 0
if debug == 1:
    DebugOutput("parallel%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

for usepot in (True, False):
    if ismaster:
        atoms = bulk('Cu')
        atoms = atoms.repeat((10, 10, 10))
        if isparallel:
            atoms = atoms.repeat(cpulayout)
        del atoms[100]   #Create point defect
    else:
        atoms = None
    if isparallel:
        atoms = MakeParallelAtoms(atoms, cpulayout)
    natoms = atoms.get_number_of_atoms()
    if usepot:
        atoms.set_calculator(EMT())
        atoms.get_potential_energy()
    
    print("Testing CNA")
    RestrictedCNA(atoms)
    t = atoms.get_tags()
    counts = np.zeros(3, int)
    for i in range(3):
        counts[i] = (t == i).sum()
    world.sum(counts)
    
    ReportTest("CNA: fcc atoms", counts[0], natoms-12, 0)
    ReportTest("CNA: hcp atoms", counts[1], 0, 0)
    ReportTest("CNA: other atoms", counts[2], 12, 0)
    
    print()
    print("Testing coordination number")
    t = CoordinationNumbers(atoms)
    counts = np.zeros(20, int)
    wanted = np.zeros(20, int)
    wanted[11] = 12
    wanted[12] = natoms - 12
    for i in range(len(counts)):
        counts[i] = (t == i).sum()
    if isparallel:
        world.sum(counts)
    for i in range(len(counts)):
        ReportTest("Coordination number {0}".format(i), counts[i], wanted[i], 0)
        
    print()
    print("Testing Full CNA")
    cnacalc = FullCNA(atoms)
    cna = cnacalc.get_normal_cna()
    print(cna[0])
    ReportTest("Length of CNA vector", len(cna), len(atoms), 0)
    for i in range(len(atoms)):
        ReportTest("Number of CNA vectors (atom {0})".format(i), 
                   sum(cna[i].values()), t[i], 0, silent=True)
        if (t[i] == 12):
            ReportTest("Number of 421 bonds (atom {0})".format(i),
                       cna[i][(4,2,1)], 12, 0, silent=True)
      
ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from ase.md.verlet import VelocityVerlet
from ase.lattice.cubic import FaceCenteredCubic
from asap3.io.trajectory import *
from numpy import *
import sys, os, time
from asap3.testtools import ReportTest
from asap3.mpi import world

debug = 0
if debug == 1:
    DebugOutput("debug%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

print_version(1)
#set_verbose(1)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

delete = True
precision = 1e-8

def maketraj(atoms, t, nstep):
    e = [atoms.get_potential_energy()]
    print("Shape of force:", atoms.get_forces().shape)
    dyn = VelocityVerlet(atoms, 5*units.fs)
    for i in range(nstep):
        dyn.run(10)
        energy = atoms.get_potential_energy()
        e.append(energy)
        if ismaster:
            print("Energy: ", energy)
        if t is not None:
            t.write()
    return e

def checktraj(t, e, cpus=None):
    i = 0
    for energy in e:
        atoms = t.get_atoms(i, cpus)
        atoms.set_calculator(EMT())
        ReportTest("Checking frame %d / cpus=%s" % (i, str(cpus)),
                   atoms.get_potential_energy(), energy, precision)
        i += 1

if ismaster:
    initial = FaceCenteredCubic(size=(10,10,10), symbol="Cu", pbc=(1,0,0))
else:
    initial = None
if isparallel:
    atoms = MakeParallelAtoms(initial, cpulayout)
else:
    atoms = initial.copy()
    
atoms.set_calculator(EMT())
atoms.set_momenta(zeros((len(atoms), 3)))
print("Writing trajectory")
traj = BundleTrajectory("traj1.bundle", "w", atoms, singleprecision=False)
traj.write()
energies = maketraj(atoms, traj, 10)
traj.close()

if ismaster:
    print("Reading trajectory (serial)")
    traj = BundleTrajectory("traj1.bundle")
    checktraj(traj, energies)

if isparallel:
    world.barrier()
    print("Reading trajectory (parallel)")
    traj = BundleTrajectory("traj1.bundle")
    checktraj(traj, energies, cpulayout)
    world.barrier()

print("Repeating simulation")
atoms = traj.get_atoms(5, cpulayout)
atoms.set_calculator(EMT())
energies2 = maketraj(atoms, None, 5)
if ismaster:
    for i in range(5):
        ReportTest("Rerun[%d]" % (i,), energies2[i], energies[i+5], precision)
traj.close()
world.barrier()

print("Appending to trajectory")
atoms = BundleTrajectory("traj1.bundle").get_atoms(-1, cpulayout)
atoms.set_calculator(EMT())
traj = BundleTrajectory("traj1.bundle", "a", atoms)
energies2 = maketraj(atoms, traj, 5)
traj.close()
world.barrier()

if ismaster:
    print("Reading longer trajectory")
    traj = BundleTrajectory("traj1.bundle")
    checktraj(traj, energies + energies2[1:])

world.barrier()
if delete:
    if ismaster:
        print("Deleting trajectory")
    BundleTrajectory.delete_bundle("traj1.bundle")
    
ReportTest.Summary()



from numpy import *
from asap3 import *
from ase.lattice.compounds import B2
from asap3.testtools import ReportTest

atoms = B2(size=(332,166,5), symbol=('Cu', 'Zr'), latticeconstant=2.7)
lj = LennardJones((29,40), 0.2*array([0.5, 1.0, 1.0, 0.5]),
                  array([1.236, 2.0, 2.0, 2.352]),
                  rCut=2.0*2.352)
atoms.set_calculator(lj)

# If the following line is included, the bug is not seen.
#atoms.get_potential_energy()
stress = atoms.get_stress()
exp_tr = 9.30356046e-02
for i in range(3):
    ReportTest("Diagonal elements of stress tensor", stress[i], exp_tr, 1e-6)
for i in range(3,6):
    ReportTest("Off-diagonal elements of stress tensor", stress[i], 0.0, 1e-8)
    
ReportTest.Summary()
